<?php
/*
        @author Clizio Merli, Italy
        @license GPL, see license.txt
*/
require_once("db_common.php");
/*! Implementation of DataWrapper for Firebird
**/
class FirebirdDBDataWrapper extends DBDataWrapper
{
        public function query($sql)
        {
                LogMaster::log("query::".$sql);

                $res = ibase_query($this->connection, $sql);
                if ($res === false)
                {
                        throw new Exception("Firebird - sql execution failed\n" . ibase_errcode() . ":". ibase_errmsg());
                }
                return $res;
        }

        protected function select_query($select,$from,$where,$sort,$start,$count)
        {
                if (!$from)
                {
                        return $select;
                }
                $sql = "SELECT " . $select . " FROM " . $from;
                if ($where) $sql .= " WHERE " . $where;
                if ($sort)  $sql .= " ORDER BY " . $sort;
                if ($start || $count)
                {
                        $sql .= " ROWS " . $start . " to " . ($start+$count);
                }
                LogMaster::log("select_query::".$sql);
                return $sql;
        }

        public function get_next($res)
        {
                $val = ibase_fetch_assoc($res);
                $str = "get_next::";
                if ($val)
                {
                        $str .= "'".implode ("','", $val)."'";
                        $val = array_change_key_case($val,CASE_LOWER);
                }
                LogMaster::log($str);
                return $val;
        }

        public function get_new_id()
        {
                $sqlid = "SELECT ID FROM LAST_INSERT_EVENT_ID";
                $qres = ibase_query($this->connection, $sqlid);
                if ($qres == false)
                {
                        LogMaster::log("get_new_id::");
                        return false;
                }
                $data = ibase_fetch_assoc($qres);
                LogMaster::log("get_new_id::".$data['ID']);
                return $data['ID'];
        }

        public function escape($data)
        {
                if (is_array($data))
                {
                        return array_map(__METHOD__, $data);
                }

                if (!empty($data) && is_string($data))
                {
                        return str_replace(array('\\', "\0", "\n", "\r", "'", '"', "\x1a"), array('\\\\', '\\0', '\\n', '\\r', "\\'", '\\"', '\\Z'), $data);
                }

                return $data;
        }

        public function tables_list()
        {
                $sql = "SELECT RDB$RELATION_NAME
                          FROM RDB$RELATIONS
                         WHERE RDB$VIEW_BLR IS NULL
                           AND (RDB$SYSTEM_FLAG IS NULL OR RDB$SYSTEM_FLAG = 0)";
                $res = ibase_query($this->connection, $sql);
                $tables = array();
                while ($table = ibase_fetch_row($res))
                {
                        $tables[] = $table['0'];
                }

                $str = "tables_list::";
                if ($tables)
                {
                        $str .= "'".implode ("','", $tables)."'";
                }
                LogMaster::log($str);
                return $tables;
        }

        public function fields_list($table) {
                $sql = "SELECT * FROM information_schema.columns WHERE table_name ='".$table."';";
                $result = pg_query($this->connection, $sql);
                $fields = array();
                $id = "";
                while ($field = pg_fetch_assoc($result)) {
                    $fields[] = $field["column_name"];
                }
                return array('fields' => $fields, 'key' => $id );

                $sql = "SELECT RDB$INDEX_SEGMENTS.RDB$FIELD_NAME AS FIELD_NAME,
                               RDB$INDICES.RDB$DESCRIPTION AS DESCRIPTION,
                               (RDB$INDEX_SEGMENTS.RDB$FIELD_POSITION + 1) AS FIELD_POSITION
                          FROM RDB$INDEX_SEGMENTS
                     LEFT JOIN RDB$INDICES ON RDB$INDICES.RDB$INDEX_NAME = RDB$INDEX_SEGMENTS.RDB$INDEX_NAME
                     LEFT JOIN RDB$RELATION_CONSTRAINTS ON RDB$RELATION_CONSTRAINTS.RDB$INDEX_NAME = RDB$INDEX_SEGMENTS.RDB$INDEX_NAME
                    WHERE UPPER(RDB$INDICES.RDB$RELATION_NAME) = '" . $table ."'";
                $result = ibase_query($this->connection, $sql);
                $fields = array();
                $ids= array();
                while ($field = ibase_fetch_assoc($result))
                {
                        $ids[] = $field['FIELD_NAME'];
                }

                $sql = "SELECT RF.RDB$RELATION_NAME AS TABLE_NAME,
                               RF.RDB$FIELD_NAME AS COLUMN_NAME,
                               CASE F.RDB$FIELD_TYPE
                                 WHEN   7 THEN 'SMALLINT'
                                 WHEN   8 THEN 'INTEGER'
                                 WHEN   9 THEN 'QUAD'
                                 WHEN  10 THEN 'FLOAT'
                                 WHEN  11 THEN 'D_FLOAT'
                                 WHEN  12 THEN 'DATE'
                                 WHEN  13 THEN 'TIME'
                                 WHEN  14 THEN 'CHAR'
                                 WHEN  16 THEN 'INT64'
                                 WHEN  27 THEN 'DOUBLE'
                                 WHEN  35 THEN 'TIMESTAMP'
                                 WHEN  37 THEN 'VARCHAR'
                                 WHEN  40 THEN 'CSTRING'
                                 WHEN 261 THEN 'BLOB'
                              END AS DATA_TYPE,
                              F.RDB$FIELD_LENGTH,
                              F.RDB$FIELD_SCALE
                         FROM RDB$FIELDS F
                         JOIN RDB$RELATION_FIELDS RF ON RF.RDB$FIELD_SOURCE = F.RDB$FIELD_NAME
                        WHERE RF.RDB$RELATION_NAME = '" . $table ."'";
                $result = ibase_query($this->connection, $sql);
                $id = "";
                $fields = array();
                while ($field = ibase_fetch_assoc($result))
                {
                        if (in_array($field['COLUMN_NAME'], $ids))
                        {
                                $id = $field['COLUMN_NAME'];
                        }
                        else
                        {
                                $fields[] = $field['COLUMN_NAME'];
                        }
                }

                $result = array("fields" => $fields, "key" => $id );
                $str = "fields_list::";
                if ($result)
                {
                        $str .= "'".implode ("','", $result)."'";
                }
                LogMaster::log($str);
                return $result;
        }
}
?>